/*
 * Date: Mon Jul 17 21:26:21 2000
 * Author: pIGpEN [ pigpen@s0ftpj.org, deadhead@sikurezza.org ]
 *
 * SoftProject 2000 - Digital Sekurity for Y2k
 * Sikurezza.org    - Italian Security MailingList
 *
 * Tested on: FreeBSD 4.0-RELEASE FreeBSD 4.0-RELEASE #26: Thu Ju i386
 *
 * All users are allowed to open raw sockets...
 * This kld disables EPERM in socket() and permits to allocate inpcb even if
 * the socket is raw and users haven't root permissions... bypassing suser()
 * in pru_attach() functions...
 *
 * If the kernel is installed with IPSEC 
 * #define IPSEC or you will have a kernel panic :)
 * else undefine it ...
 *
 * Note: My English sucks :) (My Italian too :))
 * 
 * Idea & Code for Linux by Gigi_Sull
 * Code for FreeBSD by pIGpEN / s0ftpj
 */


#include <sys/param.h>
#include <sys/module.h>
#include <sys/systm.h>
#include <sys/kernel.h>
#include <sys/protosw.h>
#include <sys/socket.h>
#include <sys/socketvar.h>

#include <net/route.h>

#include <netinet/in.h>
#include <netinet/in_systm.h>
#include <netinet/in_pcb.h>
#include <netinet/ip.h>
#include <netinet/ip_var.h>

#define IPSEC	/* see comments */

extern struct protosw	inetsw[];
extern struct inpcbinfo	ripcbinfo;


static int rip_attach 		__P((struct socket *, int, struct proc *));
static int (*old_rip_attach)	__P((struct socket *, int, struct proc *));
static int module_handler	__P((module_t, int, void *));

#define attach(x)	inetsw[ip_protox[x]].pr_usrreqs->pru_attach

static int
module_handler(module_t mod, int cmd, void *arg)
{
	int s;
	
	switch(cmd) {
 		case MOD_LOAD:
			s = splnet();
			old_rip_attach		=	attach(IPPROTO_RAW);
	 		attach(IPPROTO_RAW)	=	rip_attach;
			attach(IPPROTO_ICMP)	=	rip_attach;
			attach(IPPROTO_IGMP)	= 	rip_attach;
			attach(IPPROTO_RSVP)	=	rip_attach;
			attach(IPPROTO_IPIP)	=	rip_attach;
			attach(IPPROTO_IDP)	=	rip_attach;
			attach(0)		=	rip_attach;
			splx(s);
			break;
			
		case MOD_UNLOAD:
			s = splnet();
			attach(IPPROTO_RAW)	=	old_rip_attach;
			attach(IPPROTO_ICMP)	=	old_rip_attach;
			attach(IPPROTO_IGMP)	=	old_rip_attach;
			attach(IPPROTO_RSVP)	=	old_rip_attach;
			attach(IPPROTO_IPIP)	=	old_rip_attach;
			attach(IPPROTO_IDP)	=	old_rip_attach;
			attach(0)		=	old_rip_attach;
			splx(s);
			break;
	}
	
	return 0;
}

static moduledata_t s_raw = {
	"S_Raw",
	module_handler,
	NULL
};

DECLARE_MODULE(S_Raw, s_raw, SI_SUB_DRIVERS, SI_ORDER_MIDDLE);

extern int rip_sendspace, rip_recvspace;
	  
static int
rip_attach(struct socket *so, int proto, struct proc *p)
{
	struct inpcb *inp;
	int error, s;

	inp = sotoinpcb(so);
	if (inp)
		panic("rip_attach");
	/*
	 * if (p && (error = suser(p)) != 0)
	 * 	return error;
	 */

	error = soreserve(so, rip_sendspace, rip_recvspace);
	if (error)
		return error;
	s = splnet();
	error = in_pcballoc(so, &ripcbinfo, p);
	splx(s);
	if (error)
		return error;
	inp = (struct inpcb *)so->so_pcb;
	inp->inp_vflag |= INP_IPV4;
	inp->inp_ip_p = proto;
#ifdef IPSEC
	error = ipsec_init_policy(so, &inp->inp_sp);
	if (error != 0) {
		in_pcbdetach(inp);
		return error;
	}
#endif /*IPSEC*/
	return 0;
}
